@@ -6,7 +6,7 @@ import colorsys
 import param
 
 from ..core import util
-from ..core.data import (ArrayInterface, NdElementInterface, DictInterface)
+from ..core.data import ArrayInterface, NdElementInterface, DictInterface
 from ..core import (Dimension, NdMapping, Element2D,
                     Overlay, Element, Dataset, NdElement)
 from ..core.boundingregion import BoundingRegion, BoundingBox
@@ -396,6 +396,8 @@ class HeatMap(Dataset, Element2D):
 
 
     def _compute_raster(self):
+        if self.interface.gridded:
+            return self, np.flipud(self.dimension_values(2, flat=False))
         d1keys = self.dimension_values(0, False)
         d2keys = self.dimension_values(1, False)
         coords = [(d1, d2, np.NaN) for d1 in d1keys for d2 in d2keys]
@@ -640,6 +642,46 @@ class Image(SheetCoordinateSystem, Raster):
 
 
 
+class GridImage(Dataset, Element2D):
+    """
+    Grid interface based version of an Image, which will
+    eventually supercede the original Image implementation.
+    """
+
+    group = param.String(default='GridImage', constant=True)
+
+    kdims = param.List(default=[Dimension('x'), Dimension('y')],
+                       bounds=(2, 2))
+
+    vdims = param.List(default=[Dimension('z')], bounds=(1, 1))
+
+    def __init__(self, data, **params):
+        super(GridImage, self).__init__(data, **params)
+        (l, r), (b, t) = self.interface.range(self, 0), self.interface.range(self, 1)
+        (ys, xs) = self.dimension_values(2, flat=False).shape
+        xsampling = (float(r-l)/(xs-1))/2.
+        ysampling = (float(t-b)/(ys-1))/2.
+        l, r = l-xsampling, r+xsampling
+        b, t = b-ysampling, t+ysampling
+        self.bounds = BoundingBox(points=((l, b), (r, t)))
+
+    def range(self, dim, data_range=True):
+        dim_idx = dim if isinstance(dim, int) else self.get_dimension_index(dim)
+        dim = self.get_dimension(dim_idx)
+        if dim.range != (None, None):
+            return dim.range
+        elif dim_idx in [0, 1]:
+            l, b, r, t = self.bounds.lbrt()
+            if dim_idx:
+                drange = (b, t)
+            else:
+                drange = (l, r)
+            return drange
+        else:
+            return self.interface.range(self, dim)
+
+
+
 class RGB(Image):
     """
     An RGB element is a Image containing channel data for the the
